# -*- coding:utf-8 -*-
from Components.ActionMap import NumberActionMap, ActionMap
from Screens.Screen import Screen
from Screens.MessageBox import MessageBox
from Tools.Log import Log
from enigma import gFont, eLabel, addFont, eTimer, eConsoleAppContainer, gPixmapPtr, ePicLoad, loadPNG, getDesktop, \
    eServiceReference, iServiceInformation, iPlayableService, eListboxPythonMultiContent, RT_HALIGN_LEFT, \
    RT_HALIGN_RIGHT, RT_HALIGN_CENTER, RT_VALIGN_CENTER, RT_VALIGN_TOP, RT_WRAP, eListbox, eBackgroundFileEraser, \
    getPrevAsciiCode
from Components.Label import Label
from .primeHelper import MyAmazonSummary
from .primeHelper import _

DESKTOPSIZE = getDesktop(0).size()


class MySummary(Screen):
    def __init__(self, session, parent):
        Screen.__init__(self, session)
        self.skinName = "InfoBarPinSummary"


class PrimePinScreen(Screen):
    def __init__(self, session, amazon):
        # Skin
        if DESKTOPSIZE.width() > 1920:
            self.skin = """<screen position="0,0" backgroundColor="#00161b20" name="PrimeDreamPinScreen" size="2560,1440" >
                           <widget name="Pin" position="80,533" size="2400,80" backgroundColor="#00161b20" transparent="0" foregroundColor="#00ffffff" zPosition="2" font="AD; 60" valign="top" halign="center" />
                           <ePixmap position="993,667" size="573,133" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/AmazonPrimeDream/images/amazon_pin_573x133.png" zPosition="1" />
                           <widget name="Label0" position="1007,693" size="107,80" backgroundColor="#00161b20" transparent="0" foregroundColor="#00ffffff" zPosition="2" font="AD; 60" valign="center" halign="center" />
                           <widget name="Label1" position="1153,693" size="107,80" backgroundColor="#00161b20" transparent="0" foregroundColor="#00ffffff" zPosition="2" font="AD; 60" valign="center" halign="center" />
                           <widget name="Label2" position="1300,693" size="107,80" backgroundColor="#00161b20" transparent="0" foregroundColor="#00ffffff" zPosition="2" font="AD; 60" valign="center" halign="center" />
                           <widget name="Label3" position="1447,693" size="107,80" backgroundColor="#00161b20" transparent="0" foregroundColor="#00ffffff" zPosition="2" font="AD; 60" valign="center" halign="center" />
                           </screen>"""
        elif DESKTOPSIZE.width() == 1920:
            self.skin = """<screen position="0,0" backgroundColor="#00161b20" name="PrimeDreamPinScreen" size="1920,1080" >
                           <widget name="Pin" position="60,400" size="1800,60" backgroundColor="#00161b20" transparent="0" foregroundColor="#00ffffff" zPosition="2" font="AD; 45" valign="top" halign="center" />
                           <ePixmap position="745,500" size="430,100" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/AmazonPrimeDream/images/amazon_pin_430x100.png" zPosition="1" />
                           <widget name="Label0" position="755,520" size="80,60" backgroundColor="#00161b20" transparent="0" foregroundColor="#00ffffff" zPosition="2" font="AD; 45" valign="center" halign="center" />
                           <widget name="Label1" position="865,520" size="80,60" backgroundColor="#00161b20" transparent="0" foregroundColor="#00ffffff" zPosition="2" font="AD; 45" valign="center" halign="center" />
                           <widget name="Label2" position="975,520" size="80,60" backgroundColor="#00161b20" transparent="0" foregroundColor="#00ffffff" zPosition="2" font="AD; 45" valign="center" halign="center" />
                           <widget name="Label3" position="1085,520" size="80,60" backgroundColor="#00161b20" transparent="0" foregroundColor="#00ffffff" zPosition="2" font="AD; 45" valign="center" halign="center" />
                           </screen>"""
        else:
            self.skin = """<screen position="0,0" backgroundColor="#00161b20" name="PrimeDreamPinScreen" size="1280,720" >
                           <widget name="Pin" position="40,266" size="1200,40" backgroundColor="#00161b20" transparent="0" foregroundColor="#00ffffff" zPosition="2" font="AD; 30" valign="top" halign="center" />
                           <ePixmap position="496,333" size="286,67" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/AmazonPrimeDream/images/amazon_pin_286x67.png" zPosition="1" />
                           <widget name="Label0" position="503,346" size="53,40" backgroundColor="#00161b20" transparent="0" foregroundColor="#00ffffff" zPosition="2" font="AD; 30" valign="center" halign="center" />
                           <widget name="Label1" position="576,346" size="53,40" backgroundColor="#00161b20" transparent="0" foregroundColor="#00ffffff" zPosition="2" font="AD; 30" valign="center" halign="center" />
                           <widget name="Label2" position="650,346" size="53,40" backgroundColor="#00161b20" transparent="0" foregroundColor="#00ffffff" zPosition="2" font="AD; 30" valign="center" halign="center" />
                           <widget name="Label3" position="723,346" size="53,40" backgroundColor="#00161b20" transparent="0" foregroundColor="#00ffffff" zPosition="2" font="AD; 30" valign="center" halign="center" />
                           </screen>"""

        Screen.__init__(self, session)
        self['actions'] = ActionMap(['AmazonPrimeDream_Actions'],
                                    {'ok': self.keyOk,
                                     'cancel': self.keyCancel,
                                     'left': self.delete,
                                     'power': self.keyPower}, -1)

        self["myNumberActions"] = NumberActionMap(["InputActions"], {
            "1": self.keyNumberGlobal,
            "2": self.keyNumberGlobal,
            "3": self.keyNumberGlobal,
            "4": self.keyNumberGlobal,
            "5": self.keyNumberGlobal,
            "6": self.keyNumberGlobal,
            "7": self.keyNumberGlobal,
            "8": self.keyNumberGlobal,
            "9": self.keyNumberGlobal,
            "0": self.keyNumberGlobal
        }, -1)

        self['Pin'] = Label(_("Enter your profile PIN"))
        self['Label0'] = Label("")
        self['Label1'] = Label("")
        self['Label2'] = Label("")
        self['Label3'] = Label("")

        self.pin_text = ""

        self.amazon = amazon

    def keyOk(self):
        # validate PIN and close on success, otherwise show error and ask for other PIN
        if len(self.pin_text) > 3:
            result = self.amazon.validatePin(self.pin_text)
            if result:
                # PIN valid, no fasterExit
                self.close(True, False)
            else:
                self.pin_text = ""
                self.updatePin()
                self.session.open(MessageBox, windowTitle="Amazon Dream", text=_("Entered PIN is invalid. Try again."), type=MessageBox.TYPE_ERROR, timeout=5)

    def keyNumberGlobal(self, number):
        self.pin_text += str(number)
        if len(self.pin_text) <= 4:
            self.updatePin()

    def delete(self):
        if self.pin_text:
            if len(self.pin_text) > 1:
                self.pin_text = self.pin_text[:-1]
            else:
                self.pin_text = ""
            self.updatePin()

    def updatePin(self):
        for i in range(4):
            label = "Label" + str(i)
            self[label].setText("")
        if self.pin_text:
            for i in range(len(self.pin_text)):
                d = self.pin_text[i]
                label = "Label" + str(i)
                self[label].setText("*")

    def keyCancel(self):
        Log.i("Cancelling PIN screen")
        # PIN invalid, no fasterExit
        self.close(False, False)

    def keyPower(self):
        Log.i("Fast Exit Application request in PIN screen")
        # PIN invalid, fasterExit
        self.close(False, True)

    def createSummary(self):
        return MyAmazonSummary