#	-*-	coding:	utf-8	-*-
from enigma import gFont, getDesktop, eListboxPythonMultiContent, RT_HALIGN_LEFT, RT_HALIGN_RIGHT, RT_HALIGN_CENTER, \
    RT_VALIGN_CENTER, RT_VALIGN_TOP, RT_WRAP, ePicLoad, getPrevAsciiCode, eTimer, eLabel
from Tools.NumericalTextInput import NumericalTextInput
from Components.ActionMap import NumberActionMap
from Components.MenuList import MenuList
from Screens.Screen import Screen
from Components.MultiContent import MultiContentEntryText, MultiContentEntryText, MultiContentEntryPixmapAlphaTest, \
    MultiContentEntryPixmapAlphaBlend
from Tools.LoadPixmap import LoadPixmap
from Components.HTMLComponent import HTMLComponent
from Components.GUIComponent import GUIComponent
from Components.VariableText import VariableText
from Components.Label import Label
from Components.config import config, configfile
from Tools.Log import Log

import os

from .primeHelper import *
from .primeComponents import VideoInfoGui, VideoCategoryObservableWithSelectFrame, VideoCoverDownloader, VideoCategoryGuiList
from .AmazonHelper import SearchVideosLazyLoadObserver, AbstractObserver, UpdateEvent
from .primeMenu import PrimeMenu
from .primeSpinner import PrimeSpinner
from .primeMovieScreen import PrimeMovieScreen
from .primeSeasonScreen import PrimeSeasonScreen


class VirtualKeyBoardList(MenuList):
    def __init__(self, list, enableWrapAround=False):
        MenuList.__init__(self, list, enableWrapAround, eListboxPythonMultiContent)
        if DESKTOPSIZE.width() >= 1920:
            self.l.setFont(0, gFont('AD', 34))
            self.l.setItemHeight(68)
        else:
            self.l.setFont(0, gFont('AD', 22))
            self.l.setItemHeight(45)


class VirtualKeyBoardEntryComponent:
    def __init__(self):
        pass


class PrimeSearchScreen(Screen, PrimeMenu, PrimeSpinner):
    def __init__(self, session, amazon=None, **kwargs):
        # load skin
        if DESKTOPSIZE.width() > 1920:
            self.skin = """<screen backgroundColor="#00161b20" flags="wfNoBorder" name="AmazonPrimeDream" position="center,center" size="2560,1440" title="AmazonPrime">
                           <widget name="PrimeMenu" position="0,13" size="2560,93" foregroundColor="#00818b8b" backgroundColor="#00161b20" zPosition="2" transparent="1" enableWrapAround="1" />
                           <widget name="SearchText" position="53,133" size="2453,93" backgroundColor="#00161b20" foregroundColor="#00818b8b" zPosition="2" font="AD; 73" noWrap="1" valign="center" halign="left" />
                           <widget name="PrimeVideoList" position="0,267" size="2560,369" backgroundColor="#00161b20" zPosition="2" transparent="1" enableWrapAround="1" />
                           <widget name="text" position="736,707" size="1088,80" backgroundColor="#003a3a3a" foregroundColor="#00ffffff" zPosition="2" font="AD; 60" noWrap="1" valign="center" halign="left" />
                           <widget name="list" position="736,800" size="1088,373" backgroundColor="#00161b20" foregroundColor="#00ffffff" selectionDisabled="1" transparent="1" zPosition="3" />
                           <widget name="PrimeGuiInfo" position="53,707" size="2453,420" foregroundColor="#00ffffff" backgroundColor="#00161b20" zPosition="4" transparent="1" enableWrapAround="1" />
                           <widget name="PrimeSpinner" position="1233,673" size="93,93" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/AmazonPrimeDream/images/spinner/1.png" alphatest="blend" zPosition="99" />
                           </screen>"""
        elif DESKTOPSIZE.width() == 1920:
            self.skin = """<screen backgroundColor="#00161b20" flags="wfNoBorder" name="AmazonPrimeDream" position="center,center" size="1920,1080" title="AmazonPrime">
                           <widget name="PrimeMenu" position="0,10" size="1920,70" foregroundColor="#00818b8b" backgroundColor="#00161b20" zPosition="2" transparent="1" enableWrapAround="1" />
                           <widget name="SearchText" position="40,100" size="1840,70" backgroundColor="#00161b20" foregroundColor="#00818b8b" zPosition="2" font="AD; 55" noWrap="1" valign="center" halign="left" />
                           <widget name="PrimeVideoList" position="0,200" size="1920,277" backgroundColor="#00161b20" zPosition="2" transparent="1" enableWrapAround="1" />
                           <widget name="text" position="552,530" size="816,60" backgroundColor="#003a3a3a" foregroundColor="#00ffffff" zPosition="2" font="AD; 45" noWrap="1" valign="center" halign="left" />
                           <widget name="list" position="552,600" size="816,280" backgroundColor="#00161b20" foregroundColor="#00ffffff" selectionDisabled="1" transparent="1" zPosition="3" />
                           <widget name="PrimeGuiInfo" position="40,530" size="1840,315" foregroundColor="#00ffffff" backgroundColor="#00161b20" zPosition="4" transparent="1" enableWrapAround="1" />                  
                           <widget name="PrimeSpinner" position="925,505" size="70,70" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/AmazonPrimeDream/images/spinner/1.png" alphatest="blend" zPosition="99" />
                           </screen>
                        """
        else:
            self.skin = """<screen backgroundColor="#00161b20" flags="wfNoBorder" name="AmazonPrimeDream" position="center,center" size="1280,720" title="AmazonPrime">
                           <widget name="PrimeMenu" position="0,6" size="1280,46" foregroundColor="#00818b8b" backgroundColor="#00161b20" zPosition="2" transparent="1" enableWrapAround="1" />
                           <widget name="SearchText" position="26,66" size="1226,46" backgroundColor="#00161b20" foregroundColor="#00818b8b" zPosition="2" font="AD; 36" noWrap="1" valign="center" halign="left" />
                           <widget name="PrimeVideoList" position="0,133" size="1280,184" backgroundColor="#00161b20" zPosition="2" transparent="1" enableWrapAround="1" />
                           <widget name="text" position="368,353" size="544,40" backgroundColor="#003a3a3a" foregroundColor="#00ffffff" zPosition="2" font="AD; 30" noWrap="1" valign="center" halign="left" />
                           <widget name="list" position="368,400" size="544,186" backgroundColor="#00161b20" foregroundColor="#00ffffff" selectionDisabled="1" transparent="1" zPosition="3" />
                           <widget name="PrimeGuiInfo" position="26,353" size="1226,210" foregroundColor="#00ffffff" backgroundColor="#00161b20" zPosition="4" transparent="1" enableWrapAround="1" />
                           <widget name="PrimeSpinner" position="616,336" size="46,46" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/AmazonPrimeDream/images/spinner/1.png" alphatest="blend" zPosition="99" />
                           </screen>
                        """

        self.amazon = amazon
        Screen.__init__(self, session)
        PrimeMenu.__init__(self, text=PRIME_SEARCH_STR, index=0, amazon=self.amazon)
        PrimeSpinner.__init__(self)
        self.keys_list = []
        self.selectedKey = 0
        self.smsChar = None
        self.sms = NumericalTextInput(self.smsOK)

        self.videoCategoryObservable = VideoCategoryObservableWithSelectFrame(None)

        self.choosePrimeVideoList = VideoCategoryGuiList(self.videoCategoryObservable)
        self.choosePrimeVideoList.showTitle = False
        self['PrimeVideoList'] = self.choosePrimeVideoList

        self.searchVideosObserver = SearchVideosLazyLoadObserver(self.videoCategoryObservable, self.amazon.amazonConnection)

        self.choosePrimeGuiInfo = VideoInfoGui(self.videoCategoryObservable)
        self['PrimeGuiInfo'] = self.choosePrimeGuiInfo

        self.coverDownloader = VideoCoverDownloader(self.videoCategoryObservable)

        self.search_str = ""

        self.PrimeReloadSearchTimer = eTimer()
        self.PrimeReloadSearchTimer_conn = self.PrimeReloadSearchTimer.timeout.connect(self.checkSearchText)

        self.search_active = True
        self.faster_exit = False

        self.key_bg = LoadPixmap(
            '/usr/lib/enigma2/python/Plugins/Extensions/AmazonPrimeDream/images/search/%s/vkey_bg.png' % desksize)
        self.key_sel = LoadPixmap(
            '/usr/lib/enigma2/python/Plugins/Extensions/AmazonPrimeDream/images/search/%s/vkey_sel.png' % desksize)
        self.key_backspace = LoadPixmap(
            '/usr/lib/enigma2/python/Plugins/Extensions/AmazonPrimeDream/images/search/%s/vkey_backspace.png' % desksize)
        self.key_shift_sel = LoadPixmap(
            '/usr/lib/enigma2/python/Plugins/Extensions/AmazonPrimeDream/images/search/%s/vkey_shift_sel.png' % desksize)
        self.key_space = LoadPixmap(
            '/usr/lib/enigma2/python/Plugins/Extensions/AmazonPrimeDream/images/search/%s/vkey_space.png' % desksize)
        self.key_left = LoadPixmap(
            '/usr/lib/enigma2/python/Plugins/Extensions/AmazonPrimeDream/images/search/%s/vkey_left.png' % desksize)
        self.key_right = LoadPixmap(
            '/usr/lib/enigma2/python/Plugins/Extensions/AmazonPrimeDream/images/search/%s/vkey_right.png' % desksize)

        self.keyImages = {'BACKSPACE': self.key_backspace,
                          'SPACE': self.key_space,
                          'LEFT': self.key_left,
                          'RIGHT': self.key_right}
        self.keyImagesShift = {'BACKSPACE': self.key_backspace,
                               'SPACE': self.key_space,
                               'LEFT': self.key_left,
                               'RIGHT': self.key_right}

        self['text'] = Input(currPos=len(kwargs.get('text', '').decode('utf-8', 'ignore')), allMarked=False, **kwargs)
        self['list'] = VirtualKeyBoardList([])
        self['SearchText'] = Label("")
        self['actions'] = NumberActionMap(['OkCancelActions',
                                           'WizardActions',
                                           'ColorActions',
                                           'KeyboardInputActions',
                                           'InputBoxActions',
                                           "MenuActions",
                                           'AmazonPrimeDream_Actions',
                                           'InputAsciiActions'], {'gotAsciiCode': self.keyGotAscii,
                                                                  'ok': self.okClicked,
                                                                  'OKLong': self.okLongClicked,
                                                                  'cancel': self.exit,
                                                                  'left': self.left,
                                                                  'right': self.right,
                                                                  'up': self.up,
                                                                  'down': self.down,
                                                                  'menu': self.keyMenu,
                                                                  'deleteBackward': self.backClicked,
                                                                  'deleteForward': self.forwardClicked,
                                                                  'back': self.exit,
                                                                  'pageUp': self.cursorRight,
                                                                  'pageDown': self.cursorLeft,
                                                                  'power': self.keyPower,
                                                                  '1': self.keyNumberGlobal,
                                                                  '2': self.keyNumberGlobal,
                                                                  '3': self.keyNumberGlobal,
                                                                  '4': self.keyNumberGlobal,
                                                                  '5': self.keyNumberGlobal,
                                                                  '6': self.keyNumberGlobal,
                                                                  '7': self.keyNumberGlobal,
                                                                  '8': self.keyNumberGlobal,
                                                                  '9': self.keyNumberGlobal,
                                                                  '0': self.keyNumberGlobal}, -2)
        self.setLang()
        self.onExecBegin.append(self.setKeyboardModeAscii)
        self.onLayoutFinish.append(self.buildVirtualKeyBoard)
        self.onLayoutFinish.append(self.startSearchTimer)
        self.onClose.append(self.__onClose)

    def __onClose(self):
        self.sms.timer.stop()

    def setLang(self):
        self.keys_list = [[u'a',
                           u'b',
                           u'c',
                           u'd',
                           u'e',
                           u'f',
                           u'g',
                           u'h',
                           u'i',
                           u'j',
                           u'k',
                           u'l'
                           ],
                          [u'm',
                           u'n',
                           u'o',
                           u'p',
                           u'q',
                           u'r',
                           u's',
                           u't',
                           u'u',
                           u'v',
                           u'w',
                           u'x'],
                          [u'y',
                           u'z',
                           u'1',
                           u'2',
                           u'3',
                           u'4',
                           u'5',
                           u'6',
                           u'7',
                           u'8',
                           u'9',
                           u'0'],
                          [u'SPACE',
                           u'BACKSPACE',
                           u'LEFT',
                           u'RIGHT'
                           ]
                          ]

    def virtualKeyBoardEntryComponent(self, keys):
        # w, h = skin.parameters.get('VirtualKeyboard', (45, 45))
        if DESKTOPSIZE.width() >= 1920:
            w = 68
            h = 68
        else:
            w = 45
            h = 45
        key_bg_width = self.key_bg and self.key_bg.size().width() or w
        key_images = self.keyImagesShift or self.keyImages
        res = [keys]
        text = []
        x = 0
        for key in keys:
            png = key_images.get(key, None)
            if png:
                width = png.size().width()
                res.append(MultiContentEntryPixmapAlphaTest(pos=(x, 0), size=(width, h), png=png))
            else:
                width = key_bg_width
                res.append(MultiContentEntryPixmapAlphaTest(pos=(x, 0), size=(width, h), png=self.key_bg))
                text.append(MultiContentEntryText(pos=(x, 0), size=(width, h), font=0, text=key.encode('utf-8'),
                                                  flags=RT_HALIGN_CENTER | RT_VALIGN_CENTER))
            x += width

        return res + text

    def buildVirtualKeyBoard(self):
        self.previousSelectedKey = None
        self.list = []
        self.max_key = 0
        for keys in self.keys_list:
            self.list.append(self.virtualKeyBoardEntryComponent(keys))
            self.max_key += len(keys)

        self.max_key -= 1
        self.markSelectedKey()
        return

    def markSelectedKey(self):
        # w, h = skin.parameters.get('VirtualKeyboard', (45, 45))
        if DESKTOPSIZE.width() >= 1920:
            w = 68
            h = 68
        else:
            w = 45
            h = 45
        if self.previousSelectedKey is not None:
            self.list[self.previousSelectedKey / 12] = self.list[self.previousSelectedKey / 12][:-1]
        width = self.key_sel.size().width()
        try:
            x = self.list[self.selectedKey / 12][self.selectedKey % 12 + 1][1]
        except IndexError:
            self.selectedKey = self.max_key
            x = self.list[self.selectedKey / 12][self.selectedKey % 12 + 1][1]

        self.list[self.selectedKey / 12].append(
            MultiContentEntryPixmapAlphaTest(pos=(x, 0), size=(width, h), png=self.key_sel))
        self.previousSelectedKey = self.selectedKey
        self['list'].setList(self.list)
        return

    def backClicked(self):
        self['text'].deleteBackward()
        self.setSearchText()

    def forwardClicked(self):
        self['text'].deleteForward()
        self.setSearchText()

    def okClicked(self):
        if not self.prime_menu_show:
            if self.search_active:
                self.smsChar = None
                text = self.keys_list[self.selectedKey / 12][
                    self.selectedKey % 12].encode('UTF-8')
                Log.d("key text: --------------- %s " % text)
                if text == 'BACKSPACE':
                    self['text'].deleteBackward()
                elif text == 'SPACE':
                    self['text'].char(' '.encode('UTF-8'))
                elif text == 'LEFT':
                    self['text'].left()
                elif text == 'RIGHT':
                    self['text'].right()
                else:
                    self['text'].char(text)
                self.setSearchText()
                return
            else:
                self.videoCategoryObservable.execute(self)
        else:
            mode = self.key_ok_menu()
            if mode == "Prime":
                if config.primedream.only_prime.value:
                    config.primedream.only_prime.value = False
                else:
                    config.primedream.only_prime.value = True
                config.primedream.only_prime.save()
                configfile.save()
                self.startPrimeSpinner()
                self.amazon.getSearchResultPage(self.search_str, config.primedream.only_prime.value, self.cbReceivedSearchResultPage)
                self.down()
            else:
                self.close(mode, self.faster_exit)

    def fasterExit(self, faster_exit):
        if faster_exit:
            self.keyPower()

    def keyPower(self):
        self.faster_exit = True
        self.close("Exit", self.faster_exit)

    def okLongClicked(self):
        if not self.prime_menu_show:
            self.smsChar = None
            text = self.keys_list[self.selectedKey / 12][
                self.selectedKey % 12].encode('UTF-8')
            if text == 'BACKSPACE':
                self['text'].deleteAllChars()
                self['text'].update()
            self.setSearchText()
            return

    def ok(self):
        if not self.prime_menu_show:
            search_str = self['text'].getText().encode('UTF-8')
            self.setSearchText()
            self.startPrimeSpinner()
            self.amazon.getSearchResultPage(search_str, config.primedream.only_prime.value, self.cbReceivedSearchResultPage)
        else:
            self.close(self.key_ok_menu(), self.faster_exit)
            if self.PrimeReloadSearchTimer is not None:
                self.PrimeReloadSearchTimer.stop()

    def startSearchTimer(self):
        self.PrimeReloadSearchTimer.start(4000, True)

    def checkSearchText(self):
        search_str = self['text'].getText().encode('UTF-8')
        if not search_str == self.search_str and not search_str == "":
            self.search_str = search_str
            self.startPrimeSpinner()
            self.amazon.getSearchResultPage(search_str, config.primedream.only_prime.value, self.cbReceivedSearchResultPage)
        else:
            self.stopPrimeSpinner()
        self.PrimeReloadSearchTimer.start(4000, True)

    def cbReceivedSearchResultPage(self, page):
        self.stopPrimeSpinner()
        self.searchVideosObserver.searchString = self.search_str
        self.videoCategoryObservable.setSubject(page.categories[0])

    def setSearchText(self):
        self['SearchText'].setText(self['text'].getText().encode('UTF-8'))

    def exit(self):
        if not self.prime_menu_show:
            if self.videoCategoryObservable.showSelectFrame:
                if self.videoCategoryObservable.itemIndex is not 0:
                    self.videoCategoryObservable.setItemIndex(0)
                    return
            self.keyMenu()
        else:
            self.close("Exit", self.faster_exit)

    def keyMenu(self):
        self.key_menu()

    def cursorRight(self):
        if not self.prime_menu_show and not self.videoCategoryObservable.showSelectFrame:
            self['text'].right()

    def cursorLeft(self):
        if not self.prime_menu_show and not self.videoCategoryObservable.showSelectFrame:
            self['text'].left()

    def left(self):
        if not self.prime_menu_show:
            if self.search_active:
                self.smsChar = None
                self.selectedKey = self.selectedKey / 12 * 12 + (self.selectedKey + 11) % 12
                self.markSelectedKey()
                return
            else:
                self.videoCategoryObservable.increaseItemIndex(-1)
        else:
            self.key_left_menu()

    def right(self):
        if not self.prime_menu_show:
            if self.search_active:
                self.smsChar = None
                selectedKey = self.selectedKey / 12 * 12 + (self.selectedKey + 1) % 12
                self.selectedKey = selectedKey
                self.markSelectedKey()
                return
            else:
                self.videoCategoryObservable.increaseItemIndex(1)
        else:
            self.key_right_menu()

    def up(self):
        if not self.prime_menu_show:
            if self.search_active:
                self.smsChar = None
                if self.selectedKey < 12:
                    if self.videoCategoryObservable.getAllItems():
                        self.search_active = False
                        self.videoCategoryObservable.setShowSelectFrame(True)
                        self['list'].hide()
                        self['text'].hide()
                        self.choosePrimeGuiInfo.setVisible(True)
                    else:
                        self.key_menu()
                else:
                    self.selectedKey -= 12
                    if self.selectedKey < 0:
                        self.selectedKey = self.max_key / 12 * 12 + self.selectedKey % 12
                        if self.selectedKey > self.max_key:
                            self.selectedKey -= 12
                    self.markSelectedKey()
            elif self.videoCategoryObservable.showSelectFrame:
                self.videoCategoryObservable.setShowSelectFrame(False)
                self.key_menu()

    def down(self):
        if not self.prime_menu_show:
            if self.search_active:
                self.smsChar = None
                self.selectedKey += 12
                if self.selectedKey > self.max_key:
                    self.selectedKey %= 12
                self.markSelectedKey()
                return
            else:
                self.search_active = True
                self.videoCategoryObservable.setShowSelectFrame(False)
                self['list'].show()
                self['text'].show()
                self.choosePrimeGuiInfo.setVisible(False)
        else:
            self.key_down_menu()
            if self.videoCategoryObservable.getAllItems():
                self.videoCategoryObservable.setShowSelectFrame(True)
                self.search_active = False
                self['list'].hide()
                self['text'].hide()
                self.choosePrimeGuiInfo.setVisible(True)
            else:
                self.search_active = True
                self.videoCategoryObservable.setShowSelectFrame(False)
                self['list'].show()
                self['text'].show()
                self.choosePrimeGuiInfo.setVisible(False)

    def keyNumberGlobal(self, number):
        if not self.prime_menu_show:
            self.smsChar = self.sms.getKey(number)
            self.selectAsciiKey(self.smsChar)
            self.setSearchText()

    def smsOK(self):
        if self.smsChar and self.selectAsciiKey(self.smsChar):
            Log.d('pressing ok now')
            self.okClicked()

    def keyGotAscii(self):
        self.smsChar = None
        if self.selectAsciiKey(str(chr(getPrevAsciiCode()).encode('utf-8'))):
            self.okClicked()
        return

    def selectAsciiKey(self, char):
        if char == ' ':
            self['text'].char(' '.encode('UTF-8'))
            return False
        selkey = 0
        for keys in self.keys_list:
            for key in keys:
                if key == char:
                    self.selectedKey = selkey
                    self.markSelectedKey()
                    return True
                selkey += 1
        self.setSearchText()

        return False

    def createSummary(self):
        return MyAmazonSummary


class Input(VariableText, HTMLComponent, GUIComponent, NumericalTextInput):
    TEXT = 0
    PIN = 1
    NUMBER = 2

    def __init__(self, text='', maxSize=False, visible_width=False, type=TEXT, currPos=0, allMarked=True):
        NumericalTextInput.__init__(self, self.right)
        GUIComponent.__init__(self)
        VariableText.__init__(self)
        self.type = type
        self.allmarked = allMarked and text != '' and type != self.PIN
        self.maxSize = maxSize
        self.currPos = currPos
        self.visible_width = visible_width
        self.offset = 0
        self.overwrite = maxSize
        self.setText(text)

    def __len__(self):
        return len(self.text)

    def update(self):
        if self.visible_width:
            if self.currPos < self.offset:
                self.offset = self.currPos
            if self.currPos >= self.offset + self.visible_width:
                if self.currPos == len(self.Text):
                    self.offset = self.currPos - self.visible_width
                else:
                    self.offset = self.currPos - self.visible_width + 1
            if self.offset > 0 and self.offset + self.visible_width > len(self.Text):
                self.offset = max(0, len(self.Text) - self.visible_width)
        if self.allmarked:
            self.setMarkedPos(-2)
        else:
            self.setMarkedPos(self.currPos - self.offset)
        if self.visible_width:
            if self.type == self.PIN:
                self.text = ''
                for x in self.Text[self.offset:self.offset + self.visible_width]:
                    self.text += x == ' ' and ' ' or '*'

            else:
                self.text = self.Text[self.offset:self.offset + self.visible_width].encode('utf-8') + ' '
        elif self.type == self.PIN:
            self.text = ''
            for x in self.Text:
                self.text += x == ' ' and ' ' or '*'

        else:
            self.text = self.Text.encode('utf-8') + ' '

    def setText(self, text):
        if not len(text):
            self.currPos = 0
            self.Text = u''
        elif isinstance(text, str):
            self.Text = text.decode('utf-8', 'ignore')
        else:
            self.Text = text
        self.update()

    def getText(self):
        return self.Text.encode('utf-8')

    def createWidget(self, parent):
        if self.allmarked:
            return eLabel(parent, -2)
        else:
            return eLabel(parent, self.currPos - self.offset)

    def getSize(self):
        s = self.instance.calculateSize()
        return (s.width(), s.height())

    def markAll(self):
        self.allmarked = True
        self.update()

    def innerright(self):
        if self.allmarked:
            self.currPos = 0
            self.allmarked = False
        elif self.maxSize:
            if self.currPos < len(self.Text) - 1:
                self.currPos += 1
        elif self.currPos < len(self.Text):
            self.currPos += 1

    def right(self):
        if self.type == self.TEXT:
            self.timeout()
        self.innerright()
        self.update()

    def left(self):
        if self.type == self.TEXT:
            self.timeout()
        if self.allmarked:
            if self.maxSize:
                self.currPos = len(self.Text) - 1
            else:
                self.currPos = len(self.Text)
            self.allmarked = False
        elif self.currPos > 0:
            self.currPos -= 1
        self.update()

    def up(self):
        self.allmarked = False
        if self.type == self.TEXT:
            self.timeout()
        if self.currPos == len(self.Text) or self.Text[self.currPos] == '9' or self.Text[self.currPos] == ' ':
            newNumber = '0'
        else:
            newNumber = str(int(self.Text[self.currPos]) + 1)
        self.Text = self.Text[0:self.currPos] + newNumber + self.Text[self.currPos + 1:]
        self.update()

    def down(self):
        self.allmarked = False
        if self.type == self.TEXT:
            self.timeout()
        if self.currPos == len(self.Text) or self.Text[self.currPos] == '0' or self.Text[self.currPos] == ' ':
            newNumber = '9'
        else:
            newNumber = str(int(self.Text[self.currPos]) - 1)
        self.Text = self.Text[0:self.currPos] + newNumber + self.Text[self.currPos + 1:]
        self.update()

    def home(self):
        self.allmarked = False
        if self.type == self.TEXT:
            self.timeout()
        self.currPos = 0
        self.update()

    def end(self):
        self.allmarked = False
        if self.type == self.TEXT:
            self.timeout()
        if self.maxSize:
            self.currPos = len(self.Text) - 1
        else:
            self.currPos = len(self.Text)
        self.update()

    def insertChar(self, ch, pos=False, owr=False, ins=False):
        if isinstance(ch, str):
            ch = ch.decode('utf-8', 'ignore')
        if not pos:
            pos = self.currPos
        if ins and not self.maxSize:
            self.Text = self.Text[0:pos] + ch + self.Text[pos:]
        elif owr or self.overwrite:
            self.Text = self.Text[0:pos] + ch + self.Text[pos + 1:]
        elif self.maxSize:
            self.Text = self.Text[0:pos] + ch + self.Text[pos:-1]
        else:
            self.Text = self.Text[0:pos] + ch + self.Text[pos:]

    def deleteChar(self, pos):
        if not self.maxSize:
            self.Text = self.Text[0:pos] + self.Text[pos + 1:]
        elif self.overwrite:
            self.Text = self.Text[0:pos] + u' ' + self.Text[pos + 1:]
        else:
            self.Text = self.Text[0:pos] + self.Text[pos + 1:] + u' '

    def deleteAllChars(self):
        if self.maxSize:
            self.Text = u' ' * len(self.Text)
        else:
            self.Text = u''
        self.currPos = 0
        self.update()

    def tab(self):
        if self.type == self.TEXT:
            self.timeout()
        if self.allmarked:
            self.deleteAllChars()
            self.allmarked = False
        else:
            self.insertChar(u' ', self.currPos, False, True)
            self.innerright()
        self.update()

    def delete(self):
        if self.type == self.TEXT:
            self.timeout()
        if self.allmarked:
            self.deleteAllChars()
            self.allmarked = False
        else:
            self.deleteChar(self.currPos)
            if self.maxSize and self.overwrite:
                self.innerright()
        self.update()

    def deleteBackward(self):
        if self.type == self.TEXT:
            self.timeout()
        if self.allmarked:
            self.deleteAllChars()
            self.allmarked = False
        elif self.currPos > 0:
            self.deleteChar(self.currPos - 1)
            if not self.maxSize and self.offset > 0:
                self.offset -= 1
            self.currPos -= 1
        self.update()

    def deleteForward(self):
        if self.type == self.TEXT:
            self.timeout()
        if self.allmarked:
            self.deleteAllChars()
            self.allmarked = False
        else:
            self.deleteChar(self.currPos)
        self.update()

    def toggleOverwrite(self):
        if self.type == self.TEXT:
            self.timeout()
        self.overwrite = not self.overwrite
        self.update()

    def handleAscii(self, code):
        if self.type == self.TEXT:
            self.timeout()
        if self.allmarked:
            self.deleteAllChars()
            self.allmarked = False
        self.insertChar(chr(code), self.currPos, False, False)
        self.innerright()
        self.update()

    def number(self, number):
        if self.type == self.TEXT:
            owr = self.lastKey == number
            newChar = self.getKey(number)
        elif self.type == self.PIN or self.type == self.NUMBER:
            owr = False
            newChar = str(number)
        if self.allmarked:
            self.deleteAllChars()
            self.allmarked = False
        self.insertChar(newChar, self.currPos, owr, False)
        if self.type == self.PIN or self.type == self.NUMBER:
            self.innerright()
        self.update()

    def char(self, char):
        if self.allmarked:
            self.deleteAllChars()
            self.allmarked = False
        self.insertChar(char)
        self.innerright()
        self.update()
